// gpg-archive - gpg-ized tar using the same format as PGP's PGP Zip.
// Copyright (C) 2005 Free Software Foundation, Inc.
//
// This file is part of GnuPG.
//
// GnuPG is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// GnuPG is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

// Despite the name, PGP Zip format is actually an OpenPGP-wrapped tar
// file.  To be compatible with PGP itself, this must be a USTAR format
// tar file.  Unclear on whether there is a distinction here between
// the GNU or POSIX variant of USTAR.

//Definitions for Command constructor
function Command(cmd, args) {
    this.command = cmd;
    this.args = args;
}
Command.prototype.exec = function() {
    var cmdline = this.command + this.args;
    //WScript.Echo(cmdline);
    var wshShell = WScript.CreateObject("WScript.Shell");
    var input = "";
    var oExec = wshShell.Exec(cmdline);
    while (oExec.Status == 0 || oExec.StdOut.AtEndOfStream==false) {
        input += oExec.StdOut.Read(1);
        WScript.Sleep(100);
    }
    WScript.Echo(input);
};
//Instances
var tar = new Command("tar.exe", ""); // "tar" command
var gpg = new Command("gpg.exe", ""); // "gpg" command

// gpg-zip properties
var gpgZip = new Object();
gpgZip.version = "1.4.3";
gpgZip.usageStr = [
"Usage: gpg-zip [--help] [--version] [--encrypt] [--decrypt] [--symmetric]",
"       [--list-archive] [--output FILE] [--gpg GPG] [--gpg-args ARGS]",
"       [--tar TAR] [--tar-args ARGS] filename1 [filename2, ...]",
"       directory1 [directory2, ...]",
"",
"Encrypt or sign files into an archive."
];
gpgZip.usage = function () {
    var item = "";
    for(var i = 0; i < gpgZip.usageStr.length; i++) {
        item += gpgZip.usageStr[i] + "\n";
    }
    WScript.Echo(item);
};
gpgZip.exit = function (code) {
    if((typeof code)=="undefined") {
        code = 0;
    }
    WScript.Quit(code);
};
gpgZip.remove = function (file) {
    var objFSO = WScript.CreateObject("Scripting.FileSystemObject");
    objFSO.DeleteFile(file);
};
gpgZip.args = new Array();
for(i = 0; i < WScript.Arguments.length; i++) {
    gpgZip.args[i] = WScript.Arguments(i);
}
gpgZip.list = false;
gpgZip.create = false;
gpgZip.unpack = false;
gpgZip.outfile = "";
gpgZip.infile = "";
gpgZip.files = "";

//Analysis of arguments
for(var i = 0; i < gpgZip.args.length; i++) {
    var item = gpgZip.args[i];
    if((item=="-h") || (item=="--help") || (item.match(/^--h\w*/)!=null)) {
        gpgZip.usage();
        gpgZip.exit();
    }
    else if(item=="--list-archive") {
        gpgZip.list = true;
        gpgZip.create = false;
        gpgZip.unpack = false;
    }
    else if((item=="-e") || (item=="--encrypt")) {
        gpg.args += " --encrypt";
        gpgZip.list = false;
        gpgZip.create = true;
        gpgZip.unpack = false;
    }
    else if((item=="-d") || (item=="--decrypt")) {
        gpg.args += " --decrypt";
        gpgZip.list = false;
        gpgZip.create = false;
        gpgZip.unpack = true;
    }
    else if((item=="-c") || (item=="--symmetric")) {
        gpg.args += " --symmetric";
        gpgZip.list = false;
        gpgZip.create = true;
        gpgZip.unpack = false;
    }
    else if((item=="-s") || (item=="--sign")) {
        gpg.args += " --sign";
        gpgZip.list = false;
        gpgZip.create = true;
        gpgZip.unpack = false;
    }
    else if((item=="-r") || (item=="--recipient")) {
        i++;
        if(i < gpgZip.args.length) {
            gpg.args += " --recipient " + gpgZip.args[i];
        }
        else {
            gpgZip.usage();
            gpgZip.exit(1);
        }
    }
    else if((item=="-u") || (item=="--local-user")) {
        i++;
        if(i < gpgZip.args.length) {
            gpg.args += " --local-user " + gpgZip.args[i];
        }
        else {
            gpgZip.usage();
            gpgZip.exit(1);
        }
    }
    else if((item=="-o") || (item=="--output")) {
        i++;
        if(i < gpgZip.args.length) {
            gpgZip.outfile = gpgZip.args[i];
        }
        else {
            gpgZip.usage();
            gpgZip.exit(1);
        }
    }
    else if(item=="--version") {
        WScript.Echo("gpg-zip (GnuPG) " + gpgZip.version);
        gpgZip.exit();
    }
    else if(item=="--gpg") {
        i++;
        if(i < gpgZip.args.length) {
            gpg.command = gpgZip.args[i];
        }
        else {
            gpgZip.usage();
            gpgZip.exit(1);
        }
    }
    else if(item=="--gpg-args") {
        i++;
        if(i < gpgZip.args.length) {
            gpg.args += " " + gpgZip.args[i];
        }
        else {
            gpgZip.usage();
            gpgZip.exit(1);
        }
    }
    else if(item=="--tar") {
        i++;
        if(i < gpgZip.args.length) {
            tar.command = gpgZip.args[i];
        }
        else {
            gpgZip.usage();
            gpgZip.exit(1);
        }
    }
    else if(item=="--tar-args") {
        i++;
        if(i < gpgZip.args.length) {
            tar.args += " " + gpgZip.args[i];
        }
        else {
            gpgZip.usage();
            gpgZip.exit(1);
        }
    }
    else if(item=="--") {
        i++;
        if(i < gpgZip.args.length) {
            gpgZip.infile = gpgZip.args[i];
            if(gpgZip.outfile=="") {
                var result = gpgZip.infile.match(/(\S+)(.tar|.gpg|.asc|.pgp)(\S*)/);
                if(result!=null) {
                    gpgZip.outfile = result[1];
                }
                else {
                    gpgZip.outfile = gpgZip.infile;
                }
                if(gpgZip.create==true) {
                    gpgZip.outfile += ".gpg";
                }
                else {
                    gpgZip.outfile += ".tar";
                }
            }
            for(; i < gpgZip.args.length; i++) {
                gpgZip.files += " " + gpgZip.args[i];
            }
        }
        else {
            gpgZip.usage();
            gpgZip.exit(1);
        }
        break;
    }
    else if(item.match(/^-\w*/)!=null) {
        gpgZip.usage();
        gpgZip.exit(1);
    }
    else {
        gpgZip.infile = item;
        if(gpgZip.outfile=="") {
            var result = gpgZip.infile.match(/(\S+)(.tar|.gpg|.asc|.pgp)(\S*)/);
            if(result!=null) {
                gpgZip.outfile = result[1];
            }
            else {
                gpgZip.outfile = gpgZip.infile;
            }
            if(gpgZip.create==true) {
                gpgZip.outfile += ".gpg";
            }
            else {
                gpgZip.outfile += ".tar";
            }
        }
        for(; i < gpgZip.args.length; i++) {
            gpgZip.files += " " + gpgZip.args[i];
        }
        break;
    }
}

// create gpg-zip file
if(gpgZip.create==true) {
    var tarfile = gpgZip.infile + ".tar";
    tar.args += " -cf " + tarfile + " " + gpgZip.files;
    tar.exec();
    gpg.args += " --output " + gpgZip.outfile + " " + tarfile;
    gpg.exec();
    gpgZip.remove(tarfile);
}
// list gpg-zip file
else if(gpgZip.list==true) {
    gpg.args += " --output " + gpgZip.outfile + " " + gpgZip.infile;
    gpg.exec();
    tar.args = " -tf " + gpgZip.outfile;
    tar.exec();
    gpgZip.remove(gpgZip.outfile);
}
// unpack gpg-zip file
else if(gpgZip.unpack==true) {
    gpg.args += " --output " + gpgZip.outfile + " " + gpgZip.infile;
    gpg.exec();
    tar.args += " -xvf " + gpgZip.outfile;
    tar.exec();
    gpgZip.remove(gpgZip.outfile);
}
// no command
else {
    gpgZip.usage();
    gpgZip.exit(1);
}

//End of gpg-zip command
gpgZip.exit(0);
